#!/usr/bin/env python

"""
A handler to help with testing.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.client import Client
from imiptools.data import Object, get_address
from imiptools.handlers import Handler
from imiptools.mail import Messenger
import imip_store
import sys

class TestHandler(Handler):

    """
    A content handler for use in testing, as opposed to operating within the
    mail processing pipeline.
    """

    def __init__(self, obj, user, messenger):
        Handler.__init__(self, messenger=messenger)
        Client.__init__(self, user) # this redefines the Handler initialisation

        self.set_object(obj)

    # Action methods.

    def handle_request(self, accept):

        """
        Process the current request for the current user. Return whether any
        action was taken.
        """

        # Reply only on behalf of this user.

        attendee_attr = self.update_participation(self.obj, accept and "ACCEPTED" or "DECLINED")

        if not attendee_attr:
            return None

        # NOTE: This is a simpler form of the code in imipweb.handler.

        organiser = get_address(self.obj.get_value("ORGANIZER"))

        self.obj["ATTENDEE"] = [(self.user, attendee_attr)]
        self.update_dtstamp()
        self.set_sequence(False)

        message = self.messenger.make_outgoing_message(
            [self.obj.to_part("REPLY")],
            [organiser],
            outgoing_bcc=get_address(self.user)
            )

        return message.as_string()

# A simple main program that attempts to handle a stored request, writing the
# response message to standard output.

if __name__ == "__main__":
    try:
        nargs = 5
        accept, store_dir, user, uid, recurrenceid = (sys.argv[1:nargs+1] + [None])[:nargs]
    except ValueError:
        print >>sys.stderr, "Need 'accept' or 'decline', a store directory, user URI, event UID and optional RECURRENCE-ID."
        print >>sys.stderr, "The RECURRENCE-ID must be in the form employed by the store."
        sys.exit(1)

    store = imip_store.FileStore(store_dir)
    fragment = store.get_event(user, uid, recurrenceid)

    if not fragment:
        print >>sys.stderr, "No such event:", uid, recurrenceid
        sys.exit(1)

    obj = Object(fragment)
    handler = TestHandler(obj, user, Messenger())
    response = handler.handle_request(accept == "accept")

    if response:
        store.dequeue_request(user, uid, recurrenceid)
        print response
    else:
        sys.exit(1)

# vim: tabstop=4 expandtab shiftwidth=4
